from setuptools import setup, find_packages

setup(
    name="sklearmpi",  # Nombre del paquete
    version="0.1.1",  # Versión inicial
    author="Tu Nombre",
    author_email="tu_email@ejemplo.com",
    description="Una descripción breve de tu biblioteca",
    url="https://github.com/tu_usuario/mi_libreria",  # URL del repositorio
    packages=find_packages(),  # Encuentra automáticamente submódulos
)